////////////////////////////////////////////////////////////////////////
// Démonstrateur d'interfaçage entre un casque Neurosky Mindwave Mobile
// utilisant la technologies bluetooth comme media et une carte Arduino.
//
// Réalisation: octobre 2014, Llç  
// (CC) BY-NC-SA, Collectif Stratégies Numériques
// http://www.strat-num.fr/?Psychokinésie
//
// Arduino Bluetooth Interface with Mindwave
// Based on code example provided by NeuroSky, Inc.
//
// Need Neurosky Mindwave Mobile
// Need DFRDuino Leonardo with XBee support 
// Need HC05 Bluetooth Bee Module V2.0
////////////////////////////////////////////////////////////////////////

#define BAUDRATE1 38400  // la vitesse choisie pour le BlueBee (38400, 57600)
#define BAUDRATE  115200 // Console 
#define DEBUGOUTPUT 0 // mettre à 0 pour déactiver l'affichage exaustif sur le moniteur 

// Déclaration et remplissage du tableau...
// ...représentant les broches des LEDs
const int leds[10] = {3,4,5,6,7,8,9,10,11,12};

#define LED        13     // signal quality

#define powercontrol 10

// checksum variables
byte generatedChecksum = 0;
byte checksum = 0; 
int  payloadLength = 0;
byte payloadData[64] = {0};
// signaux
byte poorQuality = 200;
byte attention = 0;
byte meditation = 0;

// system variables
long lastReceivedPacket = 0;
boolean bigPacket = false;
boolean newData = false;

long temps = 0; //dernière base de temps
bool tic=false;

// Init
void setup() {

  int i = 0;
  for(i=0; i<10; i++)
  {
    pinMode(leds[i], OUTPUT);   // déclaration de la broche en sortie
    digitalWrite(leds[i], LOW); // éteindre tous les indicateurs
  }
  
  pinMode(LED, OUTPUT);
  
  Serial1.begin(BAUDRATE1);         // Serial INPUT from Bluetooth
  Serial.begin(BAUDRATE);           // Serial OUTPUT to console
  
  // décommentez les lignes uivantes pour debug
  //while (!Serial) ; // while the serial console output stream is not open, do nothing.
  while (!Serial1) ; // while the serial mind input stream is not open, do nothing.
  
  Serial.println("Démonstrateur d'interfaçage entre un casque Neurosky Mindwave Mobile");
  Serial.println("utilisant la technologies bluetooth comme media et une carte Arduino.");  
  Serial.println("");
  
  temps = millis(); // enregistre un top départ
}

////////////////////////////////
// Read data from Serial UART //
////////////////////////////////
byte ReadOneByte() {
  int byteRead;

  while(Serial1 && !Serial1.available());
  if(!Serial1) return 0;
  byteRead = Serial1.read();

#if DEBUGOUTPUT  
  Serial.print((char)byteRead);   // echo the same byte out the USB serial (for debuging purposes)
#endif

  return byteRead;
}

///> display attention level
///> 
void displayAttentionLevel(byte attentionLevel){
  int i; int valeur = map(attentionLevel,0,100,0,9);
  Serial.print(" AttentionLevel: "); Serial.println(valeur, DEC);
  for(i=0; i<10; i++)
  {
    if( (attentionLevel>0) && (i<=valeur))
      digitalWrite(leds[i], HIGH); //on allume la LED
    else
      digitalWrite(leds[i], LOW); //ou on éteint la LED
    }
}
 
void displayElectrodesDefault(){
  for(int i=0; i<10; i++)
  {
    if(i>3 && i<6 && tic)
      digitalWrite(leds[i], HIGH); //on allume la LED
    else
      digitalWrite(leds[i], LOW); //ou on éteint la LED
    }
} 
void displayNotPaired(){
  for(int i=0; i<10; i++)
  {
    if((i<1 || i>8) && tic)
      digitalWrite(leds[i], HIGH); //on allume la LED
    else
      digitalWrite(leds[i], LOW); //ou on éteint la LED
    }
} 

/**
 * mis à jour des valeurs :  poorQuality, attention, meditation
 */
void readDataRaw(){
  if(!Serial1 || !Serial1.available()) return; // no data ready

  // Look for sync bytes
  if(ReadOneByte() == 170 && ReadOneByte() == 170) {

      payloadLength = ReadOneByte();
      if(payloadLength > 169)                      //Payload length can not be greater than 169
          return;

      generatedChecksum = 0;        
      for(int i = 0; i < payloadLength; i++) {  
        payloadData[i] = ReadOneByte();            //Read payload into memory
        generatedChecksum += payloadData[i];
      }   

      checksum = ReadOneByte();                      //Read checksum byte from stream      
      generatedChecksum = 255 - generatedChecksum;   //Take one's compliment of generated checksum

      if(checksum == generatedChecksum) {    

        poorQuality = 200;
        attention = 0;
        meditation = 0;

        for(int i = 0; i < payloadLength; i++) {    // Parse the payload
          switch (payloadData[i]) {
          case 2:
            i++;            
            poorQuality = payloadData[i];
            bigPacket = true;            
            break;
          case 4:
            i++;
            attention = payloadData[i];                        
            break;
          case 5:
            i++;
            meditation = payloadData[i];
            break;
          case 0x80:
            i = i + 3;
            break;
          case 0x83:
            i = i + 25;      
            break;
          default:
            break;
          } // switch
        } // for loop

        if(bigPacket) {
          Serial.println("- Signal Quality: 0:all is ok, [1..199]:noise, 200:electrodes aren't contacting a person's skin.");
          Serial.println("- Attention: [0..59]:bad, [60..100]:good)");
          Serial.println("- Time since last packet: (ms)");
          Serial.print("Signal Quality: ");
          Serial.print(poorQuality, DEC);
          Serial.print(" Attention: ");
          Serial.print(attention, DEC);
          Serial.print(" Time since last packet: ");
          Serial.print(millis() - lastReceivedPacket, DEC);
          Serial.print("\n");
          lastReceivedPacket = millis();
          newData = true;
          //
          //displayAttentionLevel(attention);
        }
        bigPacket = false;        
    }
    else {
      // Checksum Error
    }  // end if else for checksum
  } // end if read 0xAA byte && end if read 0xAA byte
}

//MAIN LOOP
void loop() {
  readDataRaw();
  
  // manage signal quality level alert
  if((millis()-lastReceivedPacket)>1750) // not received packet?
  {
    //Serial.println("Pas de signal ?!");
    displayNotPaired();
    digitalWrite(LED, HIGH && tic); // big problem!
  }else if(newData == true){
    if( (poorQuality == 200) ) // electrodes aren't contacting a person's skin.
    {
      Serial.println("Les électrodes ne sont pas en contact avec la peau!");
      displayElectrodesDefault();
      digitalWrite(LED, HIGH && tic); // big problem!
    }
    else{
      displayAttentionLevel(attention);
      digitalWrite(LED, LOW); // all is ok
    }
    newData = false;
  }
  

  if((millis() - temps) > 500) //si ca fait plus de 500 ms ...
  {
    tic = !tic;
    temps = millis(); //on met à jour le temps écoulé
  }
  
  
}



